#define STEADY_DELAY (50)

#include <arduino.h>

unsigned long last_pin_change_date = 0 ;
int prev_pin_state = LOW ;
int button_state = LOW ;

/** Descr: Return true if the button on the related has been clicked, i.e.
     pressed then released since previous calls.
     This function mostly considers the state of the button the last time it
     was called and its state at the current call. It ensures that the button's
     state changed from LOW to HIGH. It also ensures that the state change
     lasted at least a long enough delay (STEADY_DELAY). This allows filtering
     the noise on the signal when the button changes its state.
    Params:
      - button_pin: number the the digital pin where the button is wired.
    Ret:
      - bool: true if the button has been pressed and released, false
      otherwise.
*/
bool button_clicked (unsigned short button_pin)
{
  /* Read the current state of the pin. */
  int pin_state = digitalRead (button_pin) ;
  /* Get the current date. */
  unsigned long current_date = millis () ;
  bool pressed = false ;

  /* Update the last change date if the pin state changed. */
  if (pin_state != prev_pin_state) last_pin_change_date = current_date ;

  /* Only check for "clic" if the pin's state change was steady long enough. */
  if ((current_date - last_pin_change_date) > STEADY_DELAY) {
    /* The current state is steady for an enough long time. It is valid. */
    if (pin_state != button_state) {
      /* The pin has a different state than the previous "button" state, so
         there was a button change. */
      button_state = pin_state ;
      /* Button is "clicked" if the pin is HIGH. */
      if (button_state == HIGH) pressed = true ;
    }
  }

  /* Update the last seen state of the pin. */
  prev_pin_state = pin_state ;
  return (pressed) ;
}
